%
%  Distance Transformation Inside Surface Object for Imaris 
%
%  Copyright Bitplane AG 2011
%  modified by Matthew J. Gastinger 1/14/2013
%
%
%  Installation:

%  -For this XTension to work:
%  
%   1)	Create a new Folder for batchable XTensions
%       a.	c:/Program Files/Bitplane/BatchXTensions
%       b.	This folder can be made anywhere, but should be in public folder
%   2)	Download XTBatchProcess.m to this folder
%   3)	Download XTDistanceTransformInsideObjectForBatch.m to this folder
%   4)	Start Imaris and Click menu tab FIJI>>OPTIONS
%       a.	Add the BatchXTensions folder to the XTension folder window
%       b.	This is necessary for the batch process option to appear in Imaris menu
%
%  
%   NOTE: This XTension will NOT appear in the Imaris menus, and will only appear 
%   in conjunction with the running of the XTBatchProcess XTension
%   
%   NOTE:  This XTension is developed for working on Windows based machines only.
%   If you want to use it on MacOS, you will have to edit the .m file save location
%   to fit Mac standards. 
%
%  Description:
%
%   This XTension will create and compute a new channel, where the voxel intensity indicates
%       the shortest distance outside of a Surface object, or from the center of a Spots object.
%       - This XTension will convert the dataset to 32 bit float type
%       - This XTension will process on first Spot or surface object in the
%       Surpass scene.
%       -This batched version will also generate a saved .IMS of the
%       processed image in C:/BitplaneBatchOutput
%
%

function XTDistanceTransformInsideObjectForBatch(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end
vImarisDataSet = vImarisApplication.GetDataSet.Clone;

%Convert dataset to 32bit float
vFloatType = vImarisDataSet.GetType.eTypeFloat;
vImarisDataSet.SetType(vFloatType);

% the user has to create a scene with some surfaces
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  %msgbox('Please create some Surfaces in the Surpass scene!');
  return;
end

% get the Surfaces
    vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vImarisApplication.GetSurpassSelection);
% search the surfaces if not previously selected
if ~vImarisApplication.GetFactory.IsSurfaces(vSurfaces)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItemSurface = vSurpassScene.GetChild(vChildIndex - 1);
        if vImarisApplication.GetFactory.IsSurfaces(vDataItemSurface)
            vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vDataItemSurface);
            vDataItemSurface=vChildIndex;
            break
        end
    end
end
if isempty(vSurfaces)
   vDataItemSurface=1618; 
end

%Get the Spots
    vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);
% search the spots if not previously selected
if ~vImarisApplication.GetFactory.IsSpots(vSpots) 
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItemSpot = vSurpassScene.GetChild(vChildIndex - 1);
        if vImarisApplication.GetFactory.IsSpots(vDataItemSpot)
            vSpots = vImarisApplication.GetFactory.ToSpots(vDataItemSpot);
            vDataItemSpot=vChildIndex;
            break
        end
    end
end
if isempty(vSpots)
    vDataItemSpot=1618;
end    

vSpotsXYZ = [];
vSpotsTime = [];
vImarisApplication.DataSetPushUndo('Distance Transform');

vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];


%If there are not surface or spot objects, Stop the XTension on this file
if vDataItemSurface==vDataItemSpot
    return;
end
%Identify if the Distance Transform will process on Spots or Surface object
%Script chooses the first Spot or Surface object in the Surpass Scene
if vDataItemSurface<vDataItemSpot
    vImarisObject = vImarisApplication.GetFactory.ToSurfaces(vSurfaces);
    vIsSurfaces=true;
    vSelection=1;
else
    vSelection=2;
    vIsSurfaces=false;
    vImarisObject = vImarisApplication.GetFactory.ToSpots(vSpots);
    vSpotsValues = vImarisObject.Get;
    vSpotsXYZ = vSpotsValues.mPositionsXYZ;
    vSpotsTime = vSpotsValues.mIndicesT;

    vInd = ones(numel(vSpotsTime), 1);
    for vDim = 1:3
        vSpotsXYZ(:, vDim) = floor((vSpotsXYZ(:, vDim) - vDataMin(vDim)) ./ ...
        (vDataMax(vDim) - vDataMin(vDim)) .* vDataSize(vDim)) + 1;
        vInd = vInd & vSpotsXYZ(:, vDim) > 0 & vSpotsXYZ(:, vDim) <= vDataSize(vDim);
    end
    vSpotsXYZ = vSpotsXYZ(vInd, :);
    vSpotsTime = vSpotsTime(vInd);
end


vProgressDisplay = waitbar(0, 'Distance Transform: Preparation');

% Create a new channel where the result will be sent
vNumberOfChannels = vImarisDataSet.GetSizeC;
vImarisDataSet.SetSizeC(vNumberOfChannels + 1);
vImarisDataSet.SetChannelName(vNumberOfChannels,['Distance to ', char(vImarisObject.GetName)]);
vImarisDataSet.SetChannelColorRGBA(vNumberOfChannels, 255*256*256);

vSizeT = vImarisDataSet.GetSizeT;
for vTime = 0:vSizeT-1;
  if vIsSurfaces
    
    % Get the mask DataSet
    vMaskDataSet = vImarisObject.GetMask( ...
      vDataMin(1), vDataMin(2), vDataMin(3), ...
      vDataMax(1), vDataMax(2), vDataMax(3), ...
      vDataSize(1), vDataSize(2), vDataSize(3), vTime);
    for vIndexZ = 1:vDataSize(3)
      vSlice = vMaskDataSet.GetDataSliceBytes(vIndexZ-1, 0, 0);
      if vSelection == 1
        vSlice = vSlice ~= 1;
      else
        vSlice = vSlice == 1;
      end
      if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
        vImarisDataSet.SetDataSliceBytes(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
        vImarisDataSet.SetDataSliceShorts(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeFloat')
        vImarisDataSet.SetDataSliceFloats(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      end
    end
    
  else % is spots
    
    vThisTime = vSpotsTime == vTime;
    vThisSpots = vSpotsXYZ(vThisTime, :);
    for vIndexZ = 1:vDataSize(3)
      vSliceSpots = vThisSpots(vThisSpots(:, 3) == vIndexZ, :);
      if isempty(vSliceSpots)
        continue;
      end
      vSlice = zeros(vDataSize(1), vDataSize(2), 'uint8');
      vSlice(vSliceSpots(:, 1) + (vSliceSpots(:, 2) - 1) * vDataSize(1)) = 1;
      if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
        vImarisDataSet.SetDataSliceBytes(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
        vImarisDataSet.SetDataSliceShorts(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      elseif strcmp(vImarisDataSet.GetType,'eTypeFloat')
        vImarisDataSet.SetDataSliceFloats(vSlice, ...
          vIndexZ-1, vNumberOfChannels, vTime);
      end
    end
    
  end

  waitbar((vTime+1)/vSizeT/2, vProgressDisplay);
end

waitbar(0.5, vProgressDisplay, 'Distance Transform: Calculation');
vImarisApplication.GetImageProcessing.DistanceTransformChannel( ...
  vImarisDataSet, vNumberOfChannels, 1, false);
waitbar(1, vProgressDisplay);

vImarisApplication.SetDataSet(vImarisDataSet);

%%
% The following MATLAB code returns the name of the dataset opened in 
% Imaris and saves file as IMS (Imaris5) format
vFileNameString = vImarisApplication.GetCurrentFileName; % returns C:/Imaris/Images/retina.ims
vFileName = char(vFileNameString);
[vOldFolder, vName, vExt] = fileparts(vFileName); % returns [C:/Imaris/Images/, retina, .ims]
vNewFileName = fullfile('c:/BitplaneBatchOutput', [vName, vExt]); % returns c:/BitplaneBatchOutput/retina.ims
vImarisApplication.FileSave(vNewFileName, '');

%%

close(vProgressDisplay);


